import re
import subprocess

from TheCodeLabs_BaseUtils.DefaultLogger import DefaultLogger
from mutagen.easyid3 import EasyID3

LOG_FORMAT = '[%(levelname)-7s] - %(asctime)s - %(message)s'
LOGGER = DefaultLogger().create_logger_if_not_exists('Mp3Converter', logFormat=LOG_FORMAT)


class Mp3Converter:
    _PATTERN_MAX_VOLUME = re.compile(r'max_volume: ([-.\d+]*)\s')

    @staticmethod
    def convert_to_mp3(inputFilePath: str, outputFilePath: str) -> None:
        output = subprocess.check_output(['ffmpeg',
                                          '-i', inputFilePath,
                                          '-af', 'volumedetect',
                                          '-vn',  # ignore video streams
                                          '-sn',  # ignore subtitle streams
                                          '-sn',  # ignore data streams
                                          '-hide_banner',  # suppress unnecessary log messages
                                          '-f', 'null',
                                          'NULL'],
                                         stderr=subprocess.STDOUT)
        output = output.decode('utf-8')
        matches = re.findall(Mp3Converter._PATTERN_MAX_VOLUME, output)
        detectedMaxVolume = None
        if matches:
            detectedMaxVolume = float(matches[0])
            LOGGER.debug(f'\t\tDetected max volume: {detectedMaxVolume}dB')

        arguments = ['ffmpeg',
                     '-i', inputFilePath,
                     '-y',  # overwrite if already exists
                     '-hide_banner',  # suppress unnecessary log messages
                     '-loglevel', 'error',
                     '-vn',  # ignore video streams
                     '-sn',  # ignore subtitle streams
                     '-sn',  # ignore data streams
                     '-ac', '2',  # 2 channels
                     '-b:a', '320k']  # 320k bitrate

        if detectedMaxVolume is not None and detectedMaxVolume < 0:
            arguments.extend(['-filter:a', f'volume={abs(detectedMaxVolume)}dB'])  # normalizes volume

        arguments.append(outputFilePath)

        LOGGER.info(f'\t\tRunning {" ".join(arguments)}')
        subprocess.check_call(arguments)

    @staticmethod
    def set_mp3_tags(filePath: str, title: str, artist: str, album: str) -> None:
        LOGGER.info(f'\t\tSetting mp3 tags for "{filePath}"')
        audio = EasyID3(filePath)
        audio['title'] = title
        audio['artist'] = artist
        audio['album'] = album
        audio.save()


if __name__ == '__main__':
    Mp3Converter.convert_to_mp3('C:/Programmierung/SpotifyBackup/recorder/1 - Bignic - tetrapod.wav',
                                'C:/Programmierung/SpotifyBackup/recorder/a.mp3')
    Mp3Converter.set_mp3_tags('C:/Programmierung/SpotifyBackup/recorder/a.mp3',
                              title='Fancy Song',
                              artist='Various Artists',
                              album='Album #1')
